//-----------------------------------------------------------------------------------
// FILE NAME: InterfazFirmware.c
// TARGET DEVICE: 	C8051Fxxx
// PROGRAMMER DEVICE:	LPC2103
// CREATED ON:          30/08/2017
// CREATED BY: 		JESUS GARCIA BARAJAS
//
// Code to program a C8051Fxxx device through its C2 interface.
//
// This software targets a LPC2103 device, though it could be easily
// ported to any device.
//
// Code assumes the following Port pin connections:
// LPC2103: | C8051Fxxx:
// P0.4    -> C2CK
// P0.5    -> C2D
// VDD     -> VDD
// GND     -> GND
//

/**********************************************************
                  		Header files
 **********************************************************/
//#include <c8051f000.h>    // SFR declarations
#include "lpc21xx.h"
#include "minilib.h"

#include <string.h>
//#include <asm/termios.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>


/**********************************************************
                       Define SECTION
**********************************************************/

// Times
#define TIMERESET 100
#define TIMEBIT 2

// Macros
#define C2D (1<<5)
#define C2CK (1<<4)
#define C2D_INPUT IO0DIR &=~(C2D);
#define C2D_OUTPUT IO0DIR |=(C2D);
#define C2D_READ ((IO0PIN & (C2D))>>5)
#define C2D_1 IO0SET |=C2D;
#define C2D_0 IO0CLR |=C2D;
#define C2CK_H IO0SET |=C2CK;
#define C2CK_L IO0CLR |=C2CK;

//Register STATUS ADDRESS 
#define FLBussy    (1<<7)
#define EError     (1<<6)
#define InBusy     (1<<1) 
#define OutReady   (1<<0)

// FLASH information
#define FLASH_SIZE 8192          // FLASH size in bytes
#define NUM_PAGES FLASH_SIZE/512 // Number of 512-byte FLASH pages


// UART
#define MAX_CHAR 16

/**********************************************************
                      GLOBAL SECTION
**********************************************************/

/**********************************************************
                      FUNTION SECTION
**********************************************************/


//-----------------------------------------------------------------------------------
// UART Control Functions
//-----------------------------------------------------------------------------------
// - Allow read and write data in the UART
//-----------------------------------------------------------------------------------
// -------------------------------------------------------
void UART0_putch(char c)
{
 while(!(U0LSR&0x20)); // mientras que U0LSR.THRE == 0 
 U0THR = c;
}
// -------------------------------------------------------
char UART0_getch()
{
 while(!(U0LSR&0x01)); // mientras que U0LSR.DR == 0
 return U0RBR;
}
// -------------------------------------------------------

void UART0_Init()
{
 U0LCR = 0x83;  // 10000011 Line Control Register
                //       .. Character length 11 = 8 bits
                //      .   Stop bits         0 = 1 bit
                //     .    Parity enable     0 = No parity
                //    .     Even parity       non sense
                //   .      Stick parity      non sense
                //  .       Break control     0 = No break
                // .        DLAB              1 = Access DLM,DLL
 U0DLL = 32;    
 U0DLM =  0;    // Baud = 4*14745600/(16* 32) = 115200
 U0LCR = 0x03;  // Same a above, but DLAB = 0 = Normal access 
 U0FCR = 0x07;  // 00000111 FIFO Control Register
                //        . FIFO enable       1 = TX&RX FIFO on 
                //       .  RX FIFO Reset     1 = Reset RX FIFO
                //      .   TX FIFO Reset     1 = Reset TX FIFO
                // ..       RX int.t trigger level 00 = 1 byte
};
// -------------------------------------------------------
/**********************************************************
                      MAIN SECTION
**********************************************************/

void main (void)
{
    unsigned int Comand,NewSFR,Address;
    unsigned int SFR=0x00;
    unsigned int i=0,j,z,h,NUM,NumPage;
    unsigned int FPDAT=0x0000;
    unsigned int Aux=0x00000000;

    
    unsigned char BufferRead[256];
    unsigned char *PTR_Read=BufferRead;
/*
    for(j=0;j<(256);j++){
                       
                      BufferRead[j]=0x00; // 
                      
    }
 */   
    //_puts("\r\n\nHello World\r\n\n");//Welcome

    for(j=0;j<6;j++){// j:Number of UART readings
                          
            h=0;         //read UART data
            do{
                BufferRead[h]=UART0_getch();
              // printf("Byte %d:%02x\n",h,BufferRead[h] );//Debug BufferRead  
                h++;
            }while (!((BufferRead[h-1]==0x0A)&&(BufferRead[h-2]==0x0D)));

            //_puts("\r\n\nBuffer de Recepción:\r\n\n");
 /*           for(i=0;i<(30);i++){
                       
                      printf("Byte %d:%02x\n",i,BufferRead[i] );//Debug BufferRead 
                      
            }*/

          

/************************* Menu for selection of the chosen command *****************************/           
            Comand=BufferRead[0];
            switch (Comand)
                    {
                        case 0:{
                            NUM=BufferRead[1];
                            Aux=((BufferRead[2]<<8)&0xFF00);
                            Address=(BufferRead[3]&0x00FF)|Aux;
                            for (i=0;i<NUM;i++){
                                BufferRead[i]=BufferRead[4+i];   
                            }
                            *PTR_Read=BufferRead;
                            PTR_Read=&BufferRead[4];
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteFlashBlock(FPDAT,PTR_Read,Address,NUM);
                           // if (z==(1)) _puts("\r\nERROR \r\n\n");
                           // if (z==(0)) _puts("\r\nEscritura correcta \r\n\n");
                            break;
                        }
                        case 1:
                            NUM=BufferRead[1];
                            Aux=((BufferRead[2]<<8)&0xFF00);
                            Address=(BufferRead[3]&0x00FF)|Aux;
                            for (i=0;i<NUM;i++){
                                BufferRead[i]=BufferRead[4+i];   
                            }
                            *PTR_Read=BufferRead;
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteFlashBlock(FPDAT,PTR_Read,Address,NUM);
                          //  if (z==(1)) _puts("\r\nERROR \r\n\n");
                          //  if (z==(0)) _puts("\r\nEscritura correcta \r\n\n");
                            break;
                        case 2:
                            *PTR_Read=BufferRead;
                            NUM=BufferRead[1];
                            Address=BufferRead[2];
                            FPDAT=InitDevice();
                            InitPI();
                            z=ReadFlashBlock(FPDAT,PTR_Read,Address,NUM);
                         //   if (z==1) _puts("\r\nERROR \r\n\n");
                         //   if (z==0) _puts("\r\nLectura correcta \r\n\n");
                            break;
                        
                        case 3:
                            NumPage=BufferRead[1];
                            FPDAT=InitDevice();
                            InitPI();
                            z=EraseFlashPage (FPDAT,NumPage);
                          //  if (z==1) _puts("\r\nERROR \r\n\n");
                          //  if (z==0) _puts("\r\nBorrado por Página correcto \r\n\n");
                            break;
                        case 4:
                            FPDAT=InitDevice();
                            InitPI();
                            z=EraseDevice (FPDAT);
                         //   if (z==1) _puts("\r\nERROR \r\n\n");
                         //   if (z==0) _puts("\r\nBorrado del dispositivo correcto \r\n\n");
                            break;
                        case 5:
                            SFR=BufferRead[1];
                            NewSFR=BufferRead[2];
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteSFR(FPDAT,SFR,NewSFR);
                         //   if (z==(1)) _puts("\r\nERROR \r\n\n");
                         //   if (z==(0)) _puts("\r\nEscritura de SFR correcta \r\n\n");
                            break;
                        case 6:
                            SFR=BufferRead[1];
                            FPDAT=InitDevice();
                            InitPI();
                            z=ReadSFR(FPDAT,SFR);
                           // if (z==(1)) _puts("\r\nERROR \r\n\n");
                           // if (z==(0)) _puts("\r\nLectura de SFR correcta \r\n\n");
                            break;
            } //switch     
        if (z==0) UART0_putch(0x0C); //Report to the Pc of completed function successfully 
            //_delay_ms(2000);
            
    }//for
    	
	while (1);//PCON=1;		// CPU idle

}

